/*
 * drivers/usb/sunxi_usb/usbc/usbc.c
 * (C) Copyright 2010-2015
 * Allwinner Technology Co., Ltd. <www.allwinnertech.com>
 * daniel, 2009.09.01
 *
 * usb common ops.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 */

#include "sunxi_usb_typedef.h"
#include "sunxi_usb_bsp.h"

#define USBC_MAX_OPEN_NUM 1
//static __usbc_otg_t usbc_otg_array[USBC_MAX_OPEN_NUM];  /* usbc internal use, in charge of USB port */
//static __fifo_info_t usbc_info_g;

/*
 * get vbus current state
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 *
 * return the current VBUS state
 */
__u32 USBC_GetVbusStatus(void)
{
	__u8 reg_val = 0;
	reg_val = USBC_Readb(USBC_REG_DEVCTL(USBC0_BASE	));
	reg_val = reg_val >> USBC_BP_DEVCTL_VBUS;
	switch(reg_val & 0x03) {
	case 0x00:
		return USBC_VBUS_STATUS_BELOW_SESSIONEND;
		//break;
	case 0x01:
		return USBC_VBUS_STATUS_ABOVE_SESSIONEND_BELOW_AVALID;
		//break;
	case 0x02:
		return USBC_VBUS_STATUS_ABOVE_AVALID_BELOW_VBUSVALID;
		//break;
	case 0x03:
		return USBC_VBUS_STATUS_ABOVE_VBUSVALID;
		//break;
	default:
		return USBC_VBUS_STATUS_BELOW_SESSIONEND;
	}
}

/*
 * select the function type, now is for host, or device
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 *
 */
void USBC_OTG_SelectMode(__u32 mode)
{
	if (mode == USBC_OTG_HOST)
	{

	} else
	{

	}
}

/*
 * get the length of data that can be read from current FIFO
 * @hUSB:     handle return by USBC_open_otg, include the key data which USBC need
 * @ep_type:  ep type, tx or rx
 *
 * return the data length that can be current read
 */
__u32 USBC_ReadLenFromFifo( __u32 ep_type)
{

	switch(ep_type) {
	case USBC_EP_TYPE_EP0:
		return USBC_Readw(USBC_REG_COUNT0(USBC0_BASE));
		//break;
	case USBC_EP_TYPE_TX:
		return 0;
		//break;
	case USBC_EP_TYPE_RX:
		return USBC_Readw(USBC_REG_RXCOUNT(USBC0_BASE));
		//break;
	default:
		return 0;
	}
}

/*
 * write data packet to fifo
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 * @fifo: fifo address
 * @cnt:  data length
 * @buff: store the data to be written
 *
 * return the lenght that successfully written
 */
__u32 USBC_WritePacket(void*fifo, __u32 cnt, void *buff)
{
	__u32 len = 0;
	__u32 i32 = 0;
	__u32 i8  = 0;
	__u8  *buf8  = 0;
	__u32 *buf32 = 0;

	//--<1>-- adjust data
	buf32 = buff;
	len   = cnt;

	i32 = len >> 2;
	i8  = len & 0x03;

	//--<2>-- deal with 4byte part
	while (i32--)
	{

		USBC_Writel(*buf32++, fifo);
	}

	//--<3>-- deal with no 4byte part
	buf8 = (__u8 *)buf32;
	while (i8--)
	{
		USBC_Writeb(*buf8++, fifo);
	}

	return len;
}

/*
 * read data from fifo
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 * @fifo: fifo address
 * @cnt:  data length
 * @buff: store the data that will be read
 *
 * return the lenght that successfully read
 */
__u32 USBC_ReadPacket( void *fifo, __u32 cnt, void *buff)
{

	__u32 len = 0;
	__u32 i32 = 0;
	__u32 i8  = 0;
	__u8  *buf8  = 0;
	__u32 *buf32 = 0;

	//--<1>-- adjust data
	buf32 = buff;
	len   = cnt;

	i32 = len >> 2;
	i8  = len & 0x03;

	//--<2>-- deal with 4byte part
	while (i32--)
	{
		*buf32++ = USBC_Readl(fifo);
	}

	//--<3>-- deal with no 4byte part
	buf8 = (__u8 *)buf32;
	while (i8--)
	{
		*buf8++ = USBC_Readb(fifo);
	}

	return len;
}

void USBC_ConfigFIFO_Base(void)
{
	u32 reg_value;

	/* config usb fifo, 8kb mode */
	reg_value = USBC_Readl(SUNXI_SRAMC_BASE + 0x04);
	reg_value &= ~(0x03 << 0);
	reg_value |= (1 << 0);
	USBC_Writel(reg_value, SUNXI_SRAMC_BASE + 0x04);
}





/*
__u32 USBC_SelectFIFO(void, __u32 ep_index)
{
	__usbc_otg_t *usbc_otg = (__usbc_otg_t *)hUSB;
	__u32 fifo = 0;

	if (usbc_otg == NULL) {
		return 0;
	}

	switch(ep_index) {
	case 0:
		fifo = USBC_REG_EPFIFO0(USBC0_BASE	);
		break;

	case 1:
		fifo = USBC_REG_EPFIFO1(USBC0_BASE	);
		break;

	case 2:
		fifo = USBC_REG_EPFIFO2(USBC0_BASE	);
		break;

	case 3:
		fifo = USBC_REG_EPFIFO3(USBC0_BASE	);
		break;

	case 4:
		fifo = USBC_REG_EPFIFO4(USBC0_BASE	);
		break;

	case 5:
		fifo = USBC_REG_EPFIFO5(USBC0_BASE	);
		break;

	default:
		fifo = 0;
	}

	return fifo;
}
*/

void *USBC_SelectFIFO(__u32 ep_index)
{
	return (void*)USBC_REG_EPFIFOx(USBC0_BASE, ep_index);
}

static void USBC_ConfigFifo_TxEp_Default(void)
{
	USBC_Writew(0x00, USBC_REG_TXFIFOAD(USBC0_BASE));
	USBC_Writeb(0x00, USBC_REG_TXFIFOSZ(USBC0_BASE));
}

/*
 * set tx ep's fifo address and size
 * @hUSB:           handle return by USBC_open_otg, include the key data which USBC need
 * @is_double_fifo: if use hardware double fifo
 * @fifo_size:      fifo size = 2 ^ fifo_size
 * @fifo_addr:      fifo start addr = fifo_addr * 8
 *
 */
static void USBC_ConfigFifo_TxEp( __u32 is_double_fifo, __u32 fifo_size, __u32 fifo_addr)
{
	__u32 temp = 0;
	__u32 size = 0;   //fifo_size = 2^ (size + 3)
	__u32 addr = 0;   //fifo_addr = addr * 8

	//--<1>-- 512 align
	temp = fifo_size + 511;
	temp &= ~511;
	temp >>= 3;
	temp >>= 1;
	while(temp) {
		size++;
		temp >>= 1;
	}

	//--<2>-- calculate addr
	addr = fifo_addr >> 3;

	//--<3>--config fifo addr
	USBC_Writew(addr, USBC_REG_TXFIFOAD(USBC0_BASE));

	//--<4>--config fifo size
	USBC_Writeb((size & 0x0f), USBC_REG_TXFIFOSZ(USBC0_BASE));
	if (is_double_fifo)
	{
		USBC_REG_set_bit_b(USBC_BP_TXFIFOSZ_DPB, USBC_REG_TXFIFOSZ(USBC0_BASE));
	}
}

static void USBC_ConfigFifo_RxEp_Default(void)
{
	USBC_Writew(0x00, USBC_REG_RXFIFOAD(USBC0_BASE));
	USBC_Writeb(0x00, USBC_REG_RXFIFOSZ(USBC0_BASE));
}

/*
 * set rx ep's fifo address and size
 * @hUSB:           handle return by USBC_open_otg, include the key data which USBC need
 * @is_double_fifo: if use hardware double fifo
 * @fifo_size:      fifo size = 2 ^ fifo_size
 * @fifo_addr:      fifo start addr = fifo_addr * 8
 *
 */
static void USBC_ConfigFifo_RxEp( __u32 is_double_fifo, __u32 fifo_size, __u32 fifo_addr)
{
	__u32 temp = 0;
	__u32 size = 0;   //fifo_size = 2 ^ (size + 3)
	__u32 addr = 0;   //fifo_addr = addr * 8

	//--<1>-- 512 align
	temp = fifo_size + 511;
	temp &= ~511;
	temp >>= 3;
	temp >>= 1;
	while(temp) {
		size++;
		temp >>= 1;
	}

	//--<2>--calculate addr
	addr = fifo_addr >> 3;

	//--<3>--config fifo addr
	USBC_Writew(addr, USBC_REG_RXFIFOAD(USBC0_BASE));

	//--<2>--config fifo size
	USBC_Writeb((size & 0x0f), USBC_REG_RXFIFOSZ(USBC0_BASE));
	if (is_double_fifo) {
		USBC_REG_set_bit_b(USBC_BP_RXFIFOSZ_DPB, USBC_REG_RXFIFOSZ(USBC0_BASE));
	}
}

/*
 * config ep's fifo addr and size
 * @hUSB:     handle return by USBC_open_otg, include the key data which USBC need
 * @ep_type:  ep type
 *
 */
void USBC_ConfigFifo_Default(__u32 ep_type)
{
	switch(ep_type) {
	case USBC_EP_TYPE_EP0:
		//not support
		break;
	case USBC_EP_TYPE_TX:
		USBC_ConfigFifo_TxEp_Default();
		break;
	case USBC_EP_TYPE_RX:
		USBC_ConfigFifo_RxEp_Default();
		break;
	default:
		break;
	}
}

/*
 * config ep's fifo addr and size
 * @hUSB:           handle return by USBC_open_otg, include the key data which USBC need
 * @ep_type:        ep type
 * @is_double_fifo: if use hardware double fifo
 * @fifo_size:      fifo size = 2 ^ fifo_size
 * @fifo_addr:      fifo start addr = fifo_addr * 8
 *
 */
void USBC_ConfigFifo(__u32 ep_type, __u32 is_double_fifo, __u32 fifo_size, __u32 fifo_addr)
{
	switch(ep_type) {
	case USBC_EP_TYPE_EP0:
		//not support
		USBC_ConfigFifo_TxEp(is_double_fifo, fifo_size, fifo_addr);
		USBC_ConfigFifo_RxEp(is_double_fifo, fifo_size, fifo_addr);
		break;
	case USBC_EP_TYPE_TX:
		USBC_ConfigFifo_TxEp(is_double_fifo, fifo_size, fifo_addr);
		break;
	case USBC_EP_TYPE_RX:
		USBC_ConfigFifo_RxEp(is_double_fifo, fifo_size, fifo_addr);
		break;
	default:
		break;
	}
}

/*
 * get the last frma number
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 *
 */
__u32 USBC_GetLastFrameNumber(void)
{
	return USBC_Readl(USBC_REG_FRNUM(USBC0_BASE	));
}

/*
 * get status of DP
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 *
 */
__u32 USBC_GetStatus_Dp(void)
{
	__u32 temp = 0;
	temp = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	temp = (temp >> USBC_BP_ISCR_EXT_DP_STATUS) & 0x01;

	return temp;
}

/*
 * get status of DM
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 *
 */
__u32 USBC_GetStatus_Dm(void)
{
	__u32 temp = 0;

	temp = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	temp = (temp >> USBC_BP_ISCR_EXT_DM_STATUS) & 0x01;

	return temp;
}

/*
 * get status of DpDm
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 *
 */
__u32 USBC_GetStatus_DpDm(void)
{
	__u32 temp = 0;
	__u32 dp = 0;
	__u32 dm = 0;


	temp = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	dp = (temp >> USBC_BP_ISCR_EXT_DP_STATUS) & 0x01;
	dm = (temp >> USBC_BP_ISCR_EXT_DM_STATUS) & 0x01;
	return ((dp << 1) | dm);

}

/*
 * get current OTG mode from vendor0's id
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 *
 * return USBC_OTG_DEVICE / USBC_OTG_HOST
 */
__u32 USBC_GetOtgMode_Form_ID(void)
{
	__u32 mode = 0;

	mode = USBC_REG_test_bit_l(USBC_BP_ISCR_MERGED_ID_STATUS, USBC_REG_ISCR(USBC0_BASE	));
	if (mode) {
		return USBC_OTG_DEVICE;
	} else {
		return USBC_OTG_HOST;
	}
}

/*
 * get current OTG mode from OTG Device's B-Device
 * @hUSB: handle return by USBC_open_otg, include the key data which USBC need
 *
 * return USBC_OTG_DEVICE / USBC_OTG_HOST
 */
__u32 USBC_GetOtgMode_Form_BDevice(void)
{
	__u32 mode = 0;

	mode = USBC_REG_test_bit_b(USBC_BP_DEVCTL_B_DEVICE, USBC_REG_DEVCTL(USBC0_BASE	));
	if (mode) {
		return USBC_OTG_DEVICE;
	} else {
		return USBC_OTG_HOST;
	}
}


/*
 * otg and hci0 Controller Shared phy in SUN50I
 * select 1:to device,0:hci
 */
void USBC_SelectPhyToDevice(void)
{
	int reg_value = 0;
	reg_value = USBC_Readl(USBC0_BASE + 0x420);
	reg_value |= (0x01);
	USBC_Writel(reg_value, (USBC0_BASE + 0x420));
}

/*
 * select the bus way for data transfer
 * @hUSB:     handle return by USBC_open_otg, include the key data which USBC need
 * @io_type:  bus type, pio or dma
 * @ep_type:  ep type, rx or tx
 * @ep_index: ep index
 *
 */
void USBC_SelectBus(__u32 io_type, __u32 ep_type, __u32 ep_index)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readb(USBC_REG_VEND0(USBC0_BASE));
	if (io_type == USBC_IO_TYPE_DMA)
	{
		if (ep_type == USBC_EP_TYPE_TX)
		{
			reg_val |= ((ep_index - 0x01) << 1) << USBC_BP_VEND0_DRQ_SEL;  //drq_sel
			reg_val |= 0x1<<USBC_BP_VEND0_BUS_SEL;   //io_dma
		}else
		{
			reg_val |= ((ep_index << 1) - 0x01) << USBC_BP_VEND0_DRQ_SEL;
			reg_val |= 0x1<<USBC_BP_VEND0_BUS_SEL;
		}
	} else
	{
		//reg_val &= ~(0x1 << USBC_BP_VEND0_DRQ_SEL);  //clear drq_sel, select pio
		reg_val &= 0x00;  // clear drq_sel, select pio
	}

	/* in 1667 1673 and later ic, FIFO_BUS_SEL bit(bit24 of reg0x40 for host/device)
	 * is fixed to 1, the hw guarantee that it's ok for cpu/inner_dma/outer_dma transfer */

//	reg_val |= 0x1<<USBC_BP_VEND0_BUS_SEL;  //for 1663 set 1: enable dma, set 0: enable fifo

	USBC_Writeb(reg_val, USBC_REG_VEND0(USBC0_BASE	));
}

/* get tx ep's interrupt flag */
static __u32 USBC_INT_TxPending(void)
{
	return (USBC_Readw(USBC_REG_INTTx(USBC0_BASE)));
}

/* clear tx ep's interrupt flag */
static void USBC_INT_ClearTxPending( __u8 ep_index)
{
	USBC_Writew((1 << ep_index), USBC_REG_INTTx(USBC0_BASE));
}

/* clear all the tx ep's interrupt flag */
static void USBC_INT_ClearTxPendingAll(void )
{
	USBC_Writew(0xffff, USBC_REG_INTTx(USBC0_BASE));
}

/* get rx ep's interrupt flag */
static __u32 USBC_INT_RxPending(void)
{
	return (USBC_Readw(USBC_REG_INTRx(USBC0_BASE)));
}

/* clear rx ep's interrupt flag */
static void USBC_INT_ClearRxPending( __u8 ep_index)
{
	USBC_Writew((1 << ep_index), USBC_REG_INTRx(USBC0_BASE));
}

/* clear all the rx ep's interrupt flag */
static void USBC_INT_ClearRxPendingAll(void)
{
	USBC_Writew(0xffff, USBC_REG_INTRx(USBC0_BASE));
}

/* open a tx ep's interrupt */
static void USBC_INT_EnableTxEp( __u8 ep_index)
{
	USBC_REG_set_bit_w(ep_index, USBC_REG_INTTxE(USBC0_BASE));
}

/* open a rx ep's interrupt */
static void USBC_INT_EnableRxEp( __u8 ep_index)
{
	USBC_REG_set_bit_w(ep_index, USBC_REG_INTRxE(USBC0_BASE));
}

/* close a tx ep's interrupt */
static void USBC_INT_DisableTxEp( __u8 ep_index)
{
	USBC_REG_clear_bit_w(ep_index, USBC_REG_INTTxE(USBC0_BASE));
}

/* close a rx ep's interrupt */
static void USBC_INT_DisableRxEp( __u8 ep_index)
{
	USBC_REG_clear_bit_w(ep_index, USBC_REG_INTRxE(USBC0_BASE));
}

/* close all tx ep's interrupt */
static void USBC_INT_DisableTxAll(void)
{
	USBC_Writew(0, USBC_REG_INTTxE(USBC0_BASE));
}

/* close all rx ep's interrupt */
static void USBC_INT_DisableRxAll(void)
{
	USBC_Writew(0, USBC_REG_INTRxE(USBC0_BASE));
}

/* get ep's interrupt flag */
__u32 USBC_INT_EpPending( __u32 ep_type)
{
	switch(ep_type) {
	case USBC_EP_TYPE_EP0:
	case USBC_EP_TYPE_TX:
		return USBC_INT_TxPending();

	case USBC_EP_TYPE_RX:
		return USBC_INT_RxPending();

	default:
		return 0;
	}
}

/* clear ep's interrupt flag */
void USBC_INT_ClearEpPending( __u32 ep_type, __u8 ep_index)
{
	switch(ep_type) {
	case USBC_EP_TYPE_EP0:
	case USBC_EP_TYPE_TX:
		USBC_INT_ClearTxPending( ep_index);
		break;

	case USBC_EP_TYPE_RX:
		USBC_INT_ClearRxPending( ep_index);
		break;

	default:
		break;
	}

	return ;
}

/* clear ep's interrupt flag */
void USBC_INT_ClearEpPendingAll( __u32 ep_type)
{
	switch(ep_type) {
	case USBC_EP_TYPE_EP0:
	case USBC_EP_TYPE_TX:
		USBC_INT_ClearTxPendingAll();
		break;

	case USBC_EP_TYPE_RX:
		USBC_INT_ClearRxPendingAll();
		break;

	default:
		break;
	}

	return ;
}

/* get usb misc's interrupt flag */
__u32 USBC_INT_MiscPending(void)
{
	return (USBC_Readb(USBC_REG_INTUSB(USBC0_BASE)));
}

/* clear usb misc's interrupt flag */
void USBC_INT_ClearMiscPending( __u32 mask)
{
	USBC_Writeb(mask, USBC_REG_INTUSB(USBC0_BASE));
}

/* clear all the usb misc's interrupt flag */
void USBC_INT_ClearMiscPendingAll(void)
{
	USBC_Writeb(0xff, USBC_REG_INTUSB(USBC0_BASE	));
}

/* open a ep's interrupt */
void USBC_INT_EnableEp(__u32 ep_type, __u8 ep_index)
{
	switch(ep_type) {
	case USBC_EP_TYPE_TX:
		USBC_INT_EnableTxEp(ep_index);
		break;

	case USBC_EP_TYPE_RX:
		USBC_INT_EnableRxEp(ep_index);
		break;

	default:
		break;
	}

}

/* open a usb misc's interrupt */
void USBC_INT_EnableUsbMiscUint(__u32 mask)
{
	__u32 reg_val;
	reg_val = USBC_Readb(USBC_REG_INTUSBE(USBC0_BASE));
	reg_val |= mask;
	USBC_Writeb(reg_val, USBC_REG_INTUSBE(USBC0_BASE));
}

/* close a ep's interrupt */
void USBC_INT_DisableEp(__u32 ep_type, __u8 ep_index)
{
	switch(ep_type) {
	case USBC_EP_TYPE_TX:
		USBC_INT_DisableTxEp( ep_index);
		break;

	case USBC_EP_TYPE_RX:
		USBC_INT_DisableRxEp( ep_index);
		break;

	default:
		break;
	}

	return;
}

/* close a usb misc's interrupt */
void USBC_INT_DisableUsbMiscUint(__u32 mask)
{
	__u32 reg_val;
	reg_val = USBC_Readb(USBC_REG_INTUSBE(USBC0_BASE));
	reg_val &= ~mask;
	USBC_Writeb(reg_val, USBC_REG_INTUSBE(USBC0_BASE));
}

/* close all ep's interrupt */
void USBC_INT_DisableEpAll(__u32 ep_type)
{
	switch(ep_type) {
	case USBC_EP_TYPE_TX:
		USBC_INT_DisableTxAll();
		break;

	case USBC_EP_TYPE_RX:
		USBC_INT_DisableRxAll();
		break;

	default:
		break;
	}

	return;
}

/* close all usb misc's interrupt */
void USBC_INT_DisableUsbMiscAll(void)
{
	USBC_Writeb(0, USBC_REG_INTUSBE(USBC0_BASE));
}

/* get current active ep */
__u32 USBC_GetActiveEp(void)
{

	return USBC_Readb(USBC_REG_EPIND(USBC0_BASE	));
}

/* set the active ep */
void USBC_SelectActiveEp(__u8 ep_index)
{
	USBC_Writeb(ep_index, USBC_REG_EPIND(USBC0_BASE	));
}

/* enhance usb transfer signal */
void USBC_EnhanceSignal(void)
{
	return;
}

/* enter TestPacket mode */
void USBC_EnterMode_TestPacket(void)
{
	USBC_REG_set_bit_b(USBC_BP_TMCTL_TEST_PACKET, USBC_REG_TMCTL(USBC0_BASE	));
}

/* enter Test_K mode */
void USBC_EnterMode_Test_K(void)
{
	USBC_REG_set_bit_b(USBC_BP_TMCTL_TEST_K, USBC_REG_TMCTL(USBC0_BASE	));
}

/* enter Test_J mode */
void USBC_EnterMode_Test_J(void)
{
	USBC_REG_set_bit_b(USBC_BP_TMCTL_TEST_J, USBC_REG_TMCTL(USBC0_BASE	));
}

/* enter Test_SE0_NAK mode */
void USBC_EnterMode_Test_SE0_NAK(void)
{
	USBC_REG_set_bit_b(USBC_BP_TMCTL_TEST_SE0_NAK, USBC_REG_TMCTL(USBC0_BASE	));
}

/* clear all test mode */
void USBC_EnterMode_Idle(void)
{
	USBC_REG_clear_bit_b(USBC_BP_TMCTL_TEST_PACKET, USBC_REG_TMCTL(USBC0_BASE	));
	USBC_REG_clear_bit_b(USBC_BP_TMCTL_TEST_K, USBC_REG_TMCTL(USBC0_BASE	));
	USBC_REG_clear_bit_b(USBC_BP_TMCTL_TEST_J, USBC_REG_TMCTL(USBC0_BASE	));
	USBC_REG_clear_bit_b(USBC_BP_TMCTL_TEST_SE0_NAK, USBC_REG_TMCTL(USBC0_BASE	));
}

/* vbus, id, dpdm, these bit is set 1 to clear, so we clear these bit when operate other bits */
static __u32 USBC_WakeUp_ClearChangeDetect(__u32 reg_val)
{
	__u32 temp = reg_val;

	temp &= ~(1 << USBC_BP_ISCR_VBUS_CHANGE_DETECT);
	temp &= ~(1 << USBC_BP_ISCR_ID_CHANGE_DETECT);
	temp &= ~(1 << USBC_BP_ISCR_DPDM_CHANGE_DETECT);

	return temp;
}

void USBC_SetWakeUp_Default(void)
{
	return;
}

void USBC_EnableIdPullUp(void)
{
	__u32 reg_val = 0;

	/* vbus, id, dpdm, these bit is set 1 to clear, so we clear these bit when operate other bits */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	reg_val |= (1 << USBC_BP_ISCR_ID_PULLUP_EN);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

void USBC_DisableIdPullUp(void)
{
	__u32 reg_val = 0;

	/* vbus, id, dpdm, these bit is set 1 to clear, so we clear these bit when operate other bits */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	reg_val &= ~(1 << USBC_BP_ISCR_ID_PULLUP_EN);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

void USBC_EnableDpDmPullUp(void)
{
	__u32 reg_val = 0;

	/* vbus, id, dpdm, these bit is set 1 to clear, so we clear these bit when operate other bits */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	reg_val |= (1 << USBC_BP_ISCR_DPDM_PULLUP_EN);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

void USBC_DisableDpDmPullUp(void)
{
	__u32 reg_val = 0;

	/* vbus, id, dpdm, these bit is set 1 to clear, so we clear these bit when operate other bits */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	reg_val &= ~(1 << USBC_BP_ISCR_DPDM_PULLUP_EN);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

static void USBC_ForceIdDisable(void)
{
	__u32 reg_val = 0;

	/* vbus, id, dpdm, these bit is set 1 to clear, so we clear these bit when operate other bits */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	reg_val &= ~(0x03 << USBC_BP_ISCR_FORCE_ID);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

static void USBC_ForceIdToLow(void)
{
	__u32 reg_val = 0;

	/* first write 00, then write 10 */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	reg_val &= ~(0x03 << USBC_BP_ISCR_FORCE_ID);
	reg_val |= (0x02 << USBC_BP_ISCR_FORCE_ID);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

static void USBC_ForceIdToHigh(void)
{
	__u32 reg_val = 0;

	/* first write 00, then write 10 */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	//reg_val &= ~(0x03 << USBC_BP_ISCR_FORCE_ID);
	reg_val |= (0x03 << USBC_BP_ISCR_FORCE_ID);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

/* force id to (id_type) */
void USBC_ForceId(__u32 id_type)
{

	switch(id_type) {
	case USBC_ID_TYPE_HOST:
		USBC_ForceIdToLow();
		break;

	case USBC_ID_TYPE_DEVICE:
		USBC_ForceIdToHigh();
		break;

	default:
		USBC_ForceIdDisable();
	}
}

static void USBC_ForceVbusValidDisable(void)
{
	__u32 reg_val = 0;

	/* first write 00, then write 10 */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	reg_val &= ~(0x03 << USBC_BP_ISCR_FORCE_VBUS_VALID);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

static void USBC_ForceVbusValidToLow(void)
{
	__u32 reg_val = 0;

	/* first write 00, then write 10 */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	reg_val &= ~(0x03 << USBC_BP_ISCR_FORCE_VBUS_VALID);
	reg_val |= (0x02 << USBC_BP_ISCR_FORCE_VBUS_VALID);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

static void USBC_ForceVbusValidToHigh(void)
{
	__u32 reg_val = 0;

	/* first write 00, then write 11 */
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE));
	reg_val &= ~(0x03 << USBC_BP_ISCR_FORCE_VBUS_VALID);
	reg_val |= (0x03 << USBC_BP_ISCR_FORCE_VBUS_VALID);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE));
}

/* force vbus valid to (id_type) */
void USBC_ForceVbusValid(__u32 vbus_type)
{
	switch(vbus_type)
	{
	case USBC_VBUS_TYPE_LOW:
		USBC_ForceVbusValidToLow();
		break;

	case USBC_VBUS_TYPE_HIGH:
		USBC_ForceVbusValidToHigh();
		break;

	default:
		USBC_ForceVbusValidDisable();
	}

}

void USBC_A_valid_InputSelect(__u32 source)
{
	return;
}

void USBC_EnableUsbLineStateBypass(void)
{
	return;
}

void USBC_DisableUsbLineStateBypass(void)
{
	return;
}

void USBC_EnableHosc(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val |= 1 << USBC_BP_ISCR_HOSC_EN;
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* forbidde Hosc */
void USBC_DisableHosc(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val &= ~(1 << USBC_BP_ISCR_HOSC_EN);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* check if vbus irq occur */
__u32 USBC_IsVbusChange(void)
{
	__u32 reg_val = 0;
	__u32 temp = 0;

	// when read the volatile bit, write 1 clear it synchronously.
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));

	temp = reg_val & (1 << USBC_BP_ISCR_VBUS_CHANGE_DETECT);

	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	reg_val |= 1 << USBC_BP_ISCR_VBUS_CHANGE_DETECT;
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));

	return temp;
}

/* check if id irq occur */
__u32 USBC_IsIdChange(void)
{
	__u32 reg_val = 0;
	__u32 temp = 0;

	// when read the volatile bit, write 1 clear it synchronously.
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));

	temp = reg_val & (1 << USBC_BP_ISCR_ID_CHANGE_DETECT);

	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	reg_val |= 1 << USBC_BP_ISCR_ID_CHANGE_DETECT;
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));

	return temp;
}

/* check if dpdm irq occur */
__u32 USBC_IsDpDmChange(void)
{
	__u32 reg_val = 0;
	__u32 temp = 0;

	// when read the volatile bit, write 1 clear it synchronously.
	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));

	temp = reg_val & (1 << USBC_BP_ISCR_DPDM_CHANGE_DETECT);

	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	reg_val |= 1 << USBC_BP_ISCR_DPDM_CHANGE_DETECT;
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));

	return temp;
}

/* disable wake irq */
void USBC_DisableWakeIrq(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val &= ~(1 << USBC_BP_ISCR_IRQ_ENABLE);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* disable vbus irq */
void USBC_DisableVbusChange(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val &= ~(1 << USBC_BP_ISCR_VBUS_CHANGE_DETECT_EN);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* disable id irq */
void USBC_DisableIdChange(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val &= ~(1 << USBC_BP_ISCR_ID_CHANGE_DETECT_EN);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* disable dpdm irq */
void USBC_DisableDpDmChange(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val &= ~(1 << USBC_BP_ISCR_DPDM_CHANGE_DETECT_EN);
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* enable wake irq */
void USBC_EnableWakeIrq(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val |= 1 << USBC_BP_ISCR_IRQ_ENABLE;
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* enable vbus irq */
void USBC_EnableVbusChange(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val |= 1 << USBC_BP_ISCR_VBUS_CHANGE_DETECT_EN;
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* enable id irq */
void USBC_EnableIdChange(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val |= 1 << USBC_BP_ISCR_ID_CHANGE_DETECT_EN;
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* enable dmdp irq */
void USBC_EnableDpDmChange(void)
{
	__u32 reg_val = 0;

	reg_val = USBC_Readl(USBC_REG_ISCR(USBC0_BASE	));
	reg_val |= 1 << USBC_BP_ISCR_DPDM_CHANGE_DETECT_EN;
	reg_val = USBC_WakeUp_ClearChangeDetect(reg_val);
	USBC_Writel(reg_val, USBC_REG_ISCR(USBC0_BASE	));
}

/* test mode, get the reg value */
__u32 USBC_TestMode_ReadReg(__u32 offset, __u32 reg_width)
{
	__u32 reg_val = 0;


	if (reg_width == 8) {
		reg_val = USBC_Readb(USBC0_BASE	 + offset);
	} else if (reg_width == 16) {
		reg_val = USBC_Readw(USBC0_BASE	 + offset);
	} else if (reg_width == 32) {
		reg_val = USBC_Readl(USBC0_BASE	 + offset);
	} else {
		reg_val = 0;
	}

	return reg_val;
}




/* USB transfer type select, read/write ops. etc */
//EXPORT_SYMBOL(USBC_OTG_SelectMode);
//
//EXPORT_SYMBOL(USBC_ReadLenFromFifo);
//EXPORT_SYMBOL(USBC_WritePacket);
//EXPORT_SYMBOL(USBC_ReadPacket);
//
//EXPORT_SYMBOL(USBC_ConfigFIFO_Base);
//EXPORT_SYMBOL(USBC_GetPortFifoStartAddr);
//EXPORT_SYMBOL(USBC_GetPortFifoSize);
//EXPORT_SYMBOL(USBC_SelectFIFO);
//EXPORT_SYMBOL(USBC_ConfigFifo_Default);
//EXPORT_SYMBOL(USBC_ConfigFifo);
//
//EXPORT_SYMBOL(USBC_SelectBus);
//
//EXPORT_SYMBOL(USBC_GetActiveEp);
//EXPORT_SYMBOL(USBC_SelectActiveEp);
//
//EXPORT_SYMBOL(USBC_EnhanceSignal);
//
//EXPORT_SYMBOL(USBC_GetLastFrameNumber);
//
//
///* usb interrupt ops */
//EXPORT_SYMBOL(USBC_INT_EpPending);
//EXPORT_SYMBOL(USBC_INT_MiscPending);
//EXPORT_SYMBOL(USBC_INT_ClearEpPending);
//EXPORT_SYMBOL(USBC_INT_ClearMiscPending);
//EXPORT_SYMBOL(USBC_INT_ClearEpPendingAll);
//EXPORT_SYMBOL(USBC_INT_ClearMiscPendingAll);
//
//EXPORT_SYMBOL(USBC_INT_EnableEp);
//EXPORT_SYMBOL(USBC_INT_EnableUsbMiscUint);
//
//EXPORT_SYMBOL(USBC_INT_DisableEp);
//EXPORT_SYMBOL(USBC_INT_DisableUsbMiscUint);
//
//EXPORT_SYMBOL(USBC_INT_DisableEpAll);
//EXPORT_SYMBOL(USBC_INT_DisableUsbMiscAll);
//
///* usb control ops */
//EXPORT_SYMBOL(USBC_GetVbusStatus);
//EXPORT_SYMBOL(USBC_GetStatus_Dp);
//EXPORT_SYMBOL(USBC_GetStatus_Dm);
//EXPORT_SYMBOL(USBC_GetStatus_DpDm);
//
//EXPORT_SYMBOL(USBC_GetOtgMode_Form_ID);
//EXPORT_SYMBOL(USBC_GetOtgMode_Form_BDevice);
//
//EXPORT_SYMBOL(USBC_SetWakeUp_Default);
//
//EXPORT_SYMBOL(USBC_EnableIdPullUp);
//EXPORT_SYMBOL(USBC_DisableIdPullUp);
//EXPORT_SYMBOL(USBC_EnableDpDmPullUp);
//EXPORT_SYMBOL(USBC_DisableDpDmPullUp);
//
//EXPORT_SYMBOL(USBC_ForceId);
//EXPORT_SYMBOL(USBC_ForceVbusValid);
//
//EXPORT_SYMBOL(USBC_A_valid_InputSelect);
//
//EXPORT_SYMBOL(USBC_EnableUsbLineStateBypass);
//EXPORT_SYMBOL(USBC_DisableUsbLineStateBypass);
//EXPORT_SYMBOL(USBC_EnableHosc);
//EXPORT_SYMBOL(USBC_DisableHosc);
//
//EXPORT_SYMBOL(USBC_IsVbusChange);
//EXPORT_SYMBOL(USBC_IsIdChange);
//EXPORT_SYMBOL(USBC_IsDpDmChange);
//
//EXPORT_SYMBOL(USBC_DisableWakeIrq);
//EXPORT_SYMBOL(USBC_DisableVbusChange);
//EXPORT_SYMBOL(USBC_DisableIdChange);
//EXPORT_SYMBOL(USBC_DisableDpDmChange);
//
//EXPORT_SYMBOL(USBC_EnableWakeIrq);
//EXPORT_SYMBOL(USBC_EnableVbusChange);
//EXPORT_SYMBOL(USBC_EnableIdChange);
//EXPORT_SYMBOL(USBC_EnableDpDmChange);
//
///* usb test ops */
//EXPORT_SYMBOL(USBC_EnterMode_TestPacket);
//EXPORT_SYMBOL(USBC_EnterMode_Test_K);
//EXPORT_SYMBOL(USBC_EnterMode_Test_J);
//EXPORT_SYMBOL(USBC_EnterMode_Test_SE0_NAK);
//EXPORT_SYMBOL(USBC_EnterMode_Idle);
//
//EXPORT_SYMBOL(USBC_TestMode_ReadReg);
//
//EXPORT_SYMBOL(USBC_open_otg);
//EXPORT_SYMBOL(USBC_close_otg);
//EXPORT_SYMBOL(USBC_init);
//EXPORT_SYMBOL(USBC_exit);

